<?php
/**
 * Sliding Messages Core Class 
 * 
 * The functions related to generating the message pool and the message design runs
 * from this class
 * 
 * @author appscreo
 * @since 3.0
 *
 */
class SlidingMarketing {
	
	private $factory = array();
	
	private static $_instance;
	
	function __construct() {
		
		$this->activate('sm-admin', 'SlidingMarketingAdmin');
		
		add_action ( 'wp_enqueue_scripts', array ($this, 'load_assets' ) );
		add_action ( 'wp_footer', array ($this, 'generate_content_in_footer' ) );
		
	}
	
	public static function getInstance() {
		if ( ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self();
		}
	
		return self::$_instance;
	}
	
	
	/**
	 * Cloning disabled
	 */
	public function __clone() {
	}
	
	/**
	 * Serialization disabled
	 */
	public function __sleep() {
	}
	
	/**
	 * De-serialization disabled
	 */
	public function __wakeup() {
	}
	
	public function activate($key, $class_name) {
		if (!isset($this->factory[$key])) {
			$this->factory[$key] = new $class_name;
		}
	}
	
	public function load_assets() {
		wp_enqueue_style ( 'slidingmessages', SLIDINGMESSAGES_PLUGIN_URL . '/assets/css/slidingmessages'.(!SLIDINGMESSAGES_DEV_MODE ? '.min' : '').'.css', array (), SLIDINGMESSAGES_VERSION );
		
		if (!slidingmarketing_get_option('deactivate_fa')) {
			wp_enqueue_style ( 'fontawesome', SLIDINGMESSAGES_PLUGIN_URL . '/assets/css/font-awesome.min.css', array (), SLIDINGMESSAGES_VERSION );
		}
		
		wp_enqueue_script ( 'jquery' );
		wp_enqueue_script ( 'slidingmessages', SLIDINGMESSAGES_PLUGIN_URL . '/assets/js/slidingmessages'.(!SLIDINGMESSAGES_DEV_MODE ? '.min' : '').'.js', array ('jquery' ), SLIDINGMESSAGES_VERSION );
		
		$additional_options = array();
		$additional_options['on_hide'] = slidingmarketing_get_option('on_hide');
		$additional_options['reshow_after_setup'] = slidingmarketing_get_option('reshow_after_setup');
		$additional_options['reshow_after_setup_timer'] = slidingmarketing_get_option('reshow_after_setup_timer');
		$additional_options['hide_mobile'] = slidingmarketing_get_option('hide_condition_mobile');
		$additional_options['hide_mobile_resolution'] = slidingmarketing_get_option('hide_condition_mobile_resolution');
		
		if (empty ( $additional_options['reshow_after_setup_timer'] ))
			$additional_options['reshow_after_setup_timer'] = 'days';
		if (! empty ( $additional_options['reshow_after_setup'] )) {
			$now = time ();
			$additional_options['reshow_after_setup'] = strtotime ( sprintf ( "+%d %s", $additional_options['reshow_after_setup'], $additional_options['reshow_after_setup_timer'] ), $now ) - $now;
		}
		
		$additional_options['post_id'] = get_the_ID();
		
		wp_add_inline_script('slidingmessages', 'var slidingmarketing = '.json_encode($additional_options).';');
		
		$custom_css_code = $this->compile_custom_css();
		if ($custom_css_code != '') {
			wp_add_inline_style('slidingmessages', $custom_css_code);
		}
	}
	
	public function compile_custom_css() {
		$active_messages = $this->get_all_messages();
		
		$style_builder = '';
		
		foreach ($active_messages as $message) {
			$style_builder .= slidingmessages_message_custom_styles($message);
		}
		
		$hide_mobile = slidingmarketing_get_option('hide_condition_mobile');
		$hide_mobile_user_resolution = slidingmarketing_get_option('hide_condition_mobile_resolution');
		
		if ($hide_mobile) {
			$mobile_break = 800;
			
			if (intval($hide_mobile_user_resolution) != 0) {
				$mobile_break = $hide_mobile_user_resolution;
			}
			
			$style_builder .= '@media screen and (max-width: '.$mobile_break.'px) {';
			$style_builder .= '.slidingmessage-open,.slidingmessage { display: none !important; opacity: 0 !important; }';
			$style_builder .= '}';
		}
		
		$style_builder .= slidingmarketing_get_option('css-custom-user-styles');
		
		return $style_builder;
	}
	
	public function generate_content_in_footer() {
		$messages = $this->get_post_messages();		
		
		foreach ($messages as $message) {
			slidingmessages_generate_message($message);
		}
	}
	
	function get_post_messages() {
		global $post;
		
		$messages = array();
		$random_message = true;
		
		
		if (isset($post)) {
			$post_manual_status = get_post_meta($post->ID, 'sm-show-manual', true);
			$post_assigned_manual = get_post_meta($post->ID, 'sm-manual-messages');
						
			if ($post_assigned_manual && $post_manual_status) {				
				
				$random_message = false;
				
				foreach ($post_assigned_manual as $post_id) {
					
					if ($this->is_deactivated_from_cookie($post_id)) {
						continue;
					}
					
					$messages[] = get_post($post_id);
				}
			}
		}
		
		if (isset($_REQUEST['message_id'])) {
			$messages = array();
			$random_message = false;
			$list = explode('-', $_REQUEST['message_id']);			
			
			foreach ($list as $message_id) {
			
				$messages[] = get_post($message_id);
			}
		}
		
		if ($random_message) {
			
			$all_messages = $this->get_active_messages(true);
			
			if (!slidingmarketing_get_option('sm-allow-multiple')) {
				if (isset($all_messages[0])) {
					$messages[] = $all_messages[0];
				}
			}
			else {
				$count = slidingmarketing_get_option('sm-allow-multiple-count');
				if (intval($count) == 0) {
					$count = 5;
				}
				
				for ($i = 0; $i < $count; $i++) {
					if (isset($all_messages[$i])) {
						$messages[] = $all_messages[$i];
					}
				}
			}
						
			// Message which are set to appear always on screen no matter of the user based setup
			$always_displayed = $this->get_always_displayed_messages();
			foreach ($always_displayed as $message) {
				$messages[] = $message;
			}

		}
		
		
		return $messages;
	}
	
	public function get_always_displayed_messages() {
		global $post;
		
		$message_list = array ();
		$query = new WP_Query ( array ('post_type' => SLIDINGMESSAGES_POSTTYPE, 'posts_per_page' => - 1 ) );
		$message_list = $query->posts ? $query->posts : array ();
		
		$customized_message_list = array ();
		foreach ( $message_list as $one ) {
			$is_paused = get_post_meta($one->ID, 'sm-paused', true);
				
			if ($is_paused) {
				continue;
			}
				
			$post_manual_status = get_post_meta($one->ID, 'sm-manual-assign', true);
				
			if ($post_manual_status) {
				continue;
			}
				
			$always_displayed = get_post_meta($one->ID, 'sm-always-displayed', true);
			if (!$always_displayed) {
				continue;
			}
			
			if ($this->is_deactivated_from_cookie($one->ID)) {
				continue;
			}
				
			$fromdate = get_post_meta($one->ID, 'sm-fromdate', true);
			$todate = get_post_meta($one->ID, 'sm-todate', true);
				
			if ($fromdate != '' || $todate != '') {
				$today = date ( "Ymd" );
					
				$fromdate = str_replace ( '-', '', $fromdate );
				$todate = str_replace ( '-', '', $todate );
					
				if (intval ( $today ) < intval ( $fromdate ) && intval ( $today ) > intval ( $todate )) {
					continue;
				}
			}
				
			$type_limit = get_post_meta($one->ID, 'sm-post-types');
				
			if ($type_limit) {
		
				$foundOne = false;
		
				foreach ($type_limit as $type) {
					if ($type == 'home') {
						if (is_home() || is_front_page()) {
							$foundOne = true;
						}
					}
					else {
						if (isset($post)) {
							if ($post->post_type == $type) {
								$foundOne = true;
							}
						}
					}
				}
		
				if (!$foundOne) {
					continue;
				}
			}
				
			$customized_message_list[] = $one;
		}
		
		return $customized_message_list;
	}
	
	public function is_deactivated_from_cookie($message_id) {
		
		// Message can't be deactivated from cookies
		$nocookies = get_post_meta($message_id, 'sm-nocookies', true);
		if ($nocookies) {
			return false;
		}
		
		$post_id = get_the_ID();
		
		if (isset($_COOKIE['sm3-'.$message_id.'-'.$post_id]) || isset($_COOKIE['sm3-'.$message_id.'-all'])) {
			return true;
		}
		else {
			return false;
		}
	}
	
	public function get_all_messages() {
		$message_list = array ();
		$query = new WP_Query ( array ('post_type' => SLIDINGMESSAGES_POSTTYPE, 'posts_per_page' => - 1 ) );
		$message_list = $query->posts ? $query->posts : array ();
		
		$customized_message_list = array ();
		foreach ( $message_list as $one ) {
			$is_paused = get_post_meta($one->ID, 'sm-paused', true);
				
			if ($is_paused) {
				continue;
			}
			
			$fromdate = get_post_meta($one->ID, 'sm-fromdate', true);
			$todate = get_post_meta($one->ID, 'sm-todate', true);
				
			if ($fromdate != '' || $todate != '') {
				$today = date ( "Ymd" );
					
				$fromdate = str_replace ( '-', '', $fromdate );
				$todate = str_replace ( '-', '', $todate );
					
				if (intval ( $today ) < intval ( $fromdate ) && intval ( $today ) > intval ( $todate )) {
					continue;
				}
			}
			
			$customized_message_list[] = $one;
		}
		
		return $customized_message_list;
	}
	
	public function get_active_messages($shuffle = false) {
		global $post;
		
		$message_list = array ();
		$query = new WP_Query ( array ('post_type' => SLIDINGMESSAGES_POSTTYPE, 'posts_per_page' => - 1 ) );
		$message_list = $query->posts ? $query->posts : array ();
		
		$customized_message_list = array ();
		
		foreach ( $message_list as $one ) {
			
			// escape paused messages
			$is_paused = get_post_meta($one->ID, 'sm-paused', true);
			
			if ($is_paused) { continue; }
			
			// post can run manually on just one page/post 
			$post_manual_status = get_post_meta($one->ID, 'sm-manual-assign', true);
			
			if ($post_manual_status) { continue; }
			
			// message will always appear no matter of randow or post display settings
			$always_displayed = get_post_meta($one->ID, 'sm-always-displayed', true);
			if ($always_displayed) { continue; }
			
			// check if deactivated by cookie (check for avoid is executed here)
			if ($this->is_deactivated_from_cookie($one->ID)) { continue; }
			
			
			// date and time rules
			$fromdate = get_post_meta($one->ID, 'sm-fromdate', true);
			$todate = get_post_meta($one->ID, 'sm-todate', true);
			
			if ($fromdate != '' || $todate != '') {
				$today = date ( "Ymd" );
			
				$fromdate = str_replace ( '-', '', $fromdate );
				$todate = str_replace ( '-', '', $todate );
			
				if (intval ( $today ) < intval ( $fromdate ) && intval ( $today ) > intval ( $todate )) {
					continue;
				} 
			}
			
			// post type limitation
			$type_limit = get_post_meta($one->ID, 'sm-post-types');			
			if ($type_limit) {

				$foundOne = false;
				
				foreach ($type_limit as $type) {
					if ($type == 'home') {
						if (is_home() || is_front_page()) { $foundOne = true; }
					}
					else {
						if (isset($post)) {
							if ($post->post_type == $type) {
								$foundOne = true;
							}
						}
					}
				}
				
				if (!$foundOne) { continue;}
			}
			
			// categories selection
			$categories_limit = get_post_meta($one->ID, 'sm-categories');
			if ($categories_limit) {
				$foundOne = false;
				
				$cat_on_archive = get_post_meta($one->ID, 'sm-archive-pages', true);
				
				foreach ($categories_limit as $cat) {
					
					if (!$cat_on_archive) {
						if (isset($post)) {
							if (has_category($cat, $post)) {
								$foundOne = true;
							}
						}
					}
					else {
						if (is_archive() && is_category($cat)) {
							$foundOne = true;
						}
					}
				}
				
				if (!$foundOne) {
					continue;
				}
			}
			
			$customized_message_list[] = $one;
		}		
		
		
		if ($shuffle) {
			shuffle ( $customized_message_list );
		}
		
		return $customized_message_list;
	}
}

global $slidingmarketing_manager;
if (!$slidingmarketing_manager) {
	$slidingmarketing_manager = SlidingMarketing::getInstance();
}